/*
	IFFCheck.cpp
	Checks to see if an a file is a valid IFF and prints out its
	structure.
	
	Leo Chan
	August 1997
*/

#include "FileIFF.h"

class IFFPrint : public BFileIFF
{
	private:
		unsigned int level;
		
	protected:
		virtual IFFP ReadFORM(IFFChunk *form);
		virtual IFFP ReadLIST(IFFChunk *list);
		virtual IFFP ReadCAT(IFFChunk *cat);
		virtual IFFP ReadPROP(IFFChunk *prop);

		virtual IFFP ReadLocalChunk(IFFChunk *local);
		virtual IFFP ReadPropertyChunk(IFFChunk *property);
		
	public:
		IFFPrint(char *path, uint32 openMode) : BFileIFF(path, openMode){level = 0;}
		void Indent();
};

void IFFPrint :: Indent()
{
	unsigned int i;
	
	for (i = 0; i < level; i++)
		printf("  ");
}

IFFP IFFPrint :: ReadFORM(IFFChunk *form)
{
	IFFChunk subchunk;
	IFFP result;
	char ID[5], *ptr;
	unsigned long grouptype;

	if (ReadAt(form->ChunkStart + 8, &grouptype, 4) != 4) return IFF_INVALID_GROUP;	
	ptr = (char *)(&grouptype);
	ID[0] = ptr[0];	ID[1] = ptr[1]; ID[2] = ptr[2]; ID[3] = ptr[3]; ID[4] = NULL;
	Indent();
	printf("FORM %d %s\n", form->ChunkSize, ID);
	level++;
		
	result = GetNextSubchunk(form, &subchunk);
	while (result == IFF_NO_ERROR)
	{
		result = ReadGroup(&subchunk);
		if (result != IFF_NO_ERROR)
		{
			// either a PROP or Local chunk
			if (subchunk.ChunkID == 'PROP')
				return IFF_INVALID_CHUNK;
			else
			{
				result = ReadLocalChunk(&subchunk);
				if (result != IFF_NO_ERROR) return result;
			}
		}

		result = GetNextSubchunk(form, &subchunk);
	}
	
	level--;
	if (result == IFF_END_OF_CHUNK)
		return IFF_NO_ERROR;
	else
		return result;
}

IFFP IFFPrint :: ReadLIST(IFFChunk *list)
{
	IFFChunk subchunk;
	bool propok = true;
	IFFP result;
	char ID[5], *ptr;
	unsigned long grouptype;

	if (ReadAt(list->ChunkStart + 8, &grouptype, 4) != 4) return IFF_INVALID_GROUP;	
	ptr = (char *)(&grouptype);
	ID[0] = ptr[0];	ID[1] = ptr[1]; ID[2] = ptr[2]; ID[3] = ptr[3]; ID[4] = NULL;
	Indent();
	printf("LIST %d %s\n", list->ChunkSize, ID);

	level++;
	result = GetNextSubchunk(list, &subchunk);
	while (result == IFF_NO_ERROR)
	{
		result = ReadGroup(&subchunk);
		if (result != IFF_NO_ERROR)
		{
			// could be a PROP chunk
			if (subchunk.ChunkID == 'PROP')
			{
				if (propok)
				{
					propok = false;
					result = ReadPROP(&subchunk);
					if (result != IFF_NO_ERROR) return result;
				}
				else
					return IFF_INVALID_CHUNK;
			}
			else
				return IFF_INVALID_CHUNK;
		}

		result = GetNextSubchunk(list, &subchunk);
	}
	
	level--;
	if (result == IFF_END_OF_CHUNK)
		return IFF_NO_ERROR;
	else
		return result;
}

IFFP IFFPrint :: ReadCAT(IFFChunk *cat)
{
	IFFChunk subchunk;
	IFFP result;
	char ID[5], *ptr;
	unsigned long grouptype;

	if (ReadAt(cat->ChunkStart + 8, &grouptype, 4) != 4) return IFF_INVALID_GROUP;	
	ptr = (char *)(&grouptype);
	ID[0] = ptr[0];	ID[1] = ptr[1]; ID[2] = ptr[2]; ID[3] = ptr[3]; ID[4] = NULL;
	Indent();
	printf("CAT  %d %s\n", cat->ChunkSize, ID);

	level++;		
	result = GetNextSubchunk(cat, &subchunk);
	while (result == IFF_NO_ERROR)
	{
		result = ReadGroup(&subchunk);
		if (result != IFF_NO_ERROR) return result;

		result = GetNextSubchunk(cat, &subchunk);
	}
	
	level--;
	if (result == IFF_END_OF_CHUNK)
		return IFF_NO_ERROR;
	else
		return result;
}

IFFP IFFPrint :: ReadPROP(IFFChunk *prop)
{
	IFFChunk subchunk;
	IFFP result;
	char ID[5], *ptr;
	unsigned long grouptype;

	if (ReadAt(prop->ChunkStart + 8, &grouptype, 4) != 4) return IFF_INVALID_GROUP;	
	ptr = (char *)(&grouptype);
	ID[0] = ptr[0];	ID[1] = ptr[1]; ID[2] = ptr[2]; ID[3] = ptr[3]; ID[4] = NULL;
	Indent();
	printf("PROP %d %s\n", prop->ChunkSize, ID);
	
	level++;
	result = GetNextSubchunk(prop, &subchunk);
	while (result == IFF_NO_ERROR)
	{
		if (subchunk.ChunkID == 'FORM' || subchunk.ChunkID == 'LIST'
				|| subchunk.ChunkID == 'CAT ' || subchunk.ChunkID == 'PROP')
			return false;
			
		if (!ReadPropertyChunk(&subchunk)) return false;

		result = GetNextSubchunk(prop, &subchunk);
	}
	
	level--;
	if (result == IFF_END_OF_CHUNK)
		return IFF_NO_ERROR;
	else
		return result;
}

IFFP IFFPrint :: ReadLocalChunk(IFFChunk *local)
{
	char ID[5], *ptr;
	
	ptr = (char *)(&(local->ChunkID));
	ID[0] = ptr[0];	ID[1] = ptr[1]; ID[2] = ptr[2]; ID[3] = ptr[3]; ID[4] = NULL;
	Indent();
	printf("%s %d\n", ID, local->ChunkSize);
		
	return IFF_NO_ERROR;
}

IFFP IFFPrint :: ReadPropertyChunk(IFFChunk *property)
{
	char ID[5], *ptr;
	
	ptr = (char *)(&(property->ChunkID));
	ID[0] = ptr[0];	ID[1] = ptr[1]; ID[2] = ptr[2]; ID[3] = ptr[3]; ID[4] = NULL;
	Indent();
	printf("%s %d\n", ID, property->ChunkSize);
		
	return IFF_NO_ERROR;
}

void main(int argc, char *argv[])
{
	IFFPrint *file;
	
	if (argc != 2)
	{
		printf("usage: %s [IFF file]\n", argv[0]);
		return;
	}
	
	file = new IFFPrint(argv[1], B_READ_ONLY);

	if (file->ReadIFF() != IFF_NO_ERROR)
		printf("\nThis is not a valid IFF file!\n");	
}
